﻿using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace RCM {
    [DisplayName("Browse via SCP")]
    public class SCPAction : IAction {
        private SSH _ssh;
        private SSHConfig _config;

        public SCPAction(SSH ssh) {
            _ssh = ssh;
            _config = (SSHConfig)Config.ConfigSections[typeof(SSHConfig)];
        }

        public async void Run() {
            await Task.Run(() => {
                if (Environment.OSVersion.Platform == PlatformID.Unix) {
                    RunUnix();
                } else {
                    RunWin();
                }
            });
        }

        private void RunWin() {
            if (string.IsNullOrWhiteSpace(_config.WinSCPPath)) {
                MessageBox.Show("WinSCP path is not set", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!File.Exists(_config.WinSCPPath)) {
                MessageBox.Show("WinSCP path does not exist", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrWhiteSpace(_config.SSHKeys) && string.IsNullOrWhiteSpace(_ssh.Password)) {
                MessageBox.Show("No authentication method was set", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!string.IsNullOrWhiteSpace(_config.SSHKeys)) {
                if (string.IsNullOrWhiteSpace(_config.PageantPath)) {
                    MessageBox.Show("Pageant path is not set", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                if (!File.Exists(_config.PageantPath)) {
                    MessageBox.Show("Pageant path does not exist", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }

            StringBuilder scpParams = new StringBuilder();
            if (!string.IsNullOrWhiteSpace(_config.SSHKeys)) {
                Process.Start(_config.PageantPath, _config.SSHKeys);
                scpParams.Append(string.Format("/privatekey={0} ", _config.SSHKeys)); //TODO: Zvlada vic klicu?
            }
            scpParams.AppendFormat("scp://{0}@{1}", _ssh.Username, _ssh.Host); // TODO: password?
            Process.Start(_config.WinSCPPath, scpParams.ToString());
        }

        private void RunUnix() {
            // TOOD
        }
    }
}
