﻿using System;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace RCM {
    [DisplayName("Start SSH")]
    public class SSHAction : IAction {
        private SSH _ssh;
        private SSHConfig _config;

        public SSHAction(SSH ssh) {
            _ssh = ssh;
            _config = (SSHConfig)Config.ConfigSections[typeof(SSHConfig)];
        }

        public async void Run() {
            await Task.Run(() => {
                if (Environment.OSVersion.Platform == PlatformID.Unix) {
                    RunUnix();
                } else {
                    RunWin();
                }
            });
        }

        private void RunWin() {
            if (string.IsNullOrWhiteSpace(_config.PuttyPath)) {
                MessageBox.Show("PuTTY path is not set", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!File.Exists(_config.PuttyPath)) {
                MessageBox.Show("PuTTY path does not exist", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrWhiteSpace(_config.SSHKeys) && string.IsNullOrWhiteSpace(_ssh.Password)) {
                MessageBox.Show("No authentication method was set", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!string.IsNullOrWhiteSpace(_config.SSHKeys)) {
                if (string.IsNullOrWhiteSpace(_config.PageantPath)) {
                    MessageBox.Show("Pageant path is not set", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                if (!File.Exists(_config.PageantPath)) {
                    MessageBox.Show("Pageant path does not exist", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }

            string[] host = _ssh.Host.Split(new char[] { ':' });
            StringBuilder puttyParams = new StringBuilder();
            if (!string.IsNullOrWhiteSpace(_config.SSHKeys)) {
                Process.Start(_config.PageantPath, _config.SSHKeys);
                puttyParams.AppendFormat("-i {0} ", _config.SSHKeys);
            } else {
                puttyParams.AppendFormat("-pw {0} ", _ssh.Password);
            }
            if (host.Length > 1) {
                puttyParams.AppendFormat("-P {0} ", host[1]);
            }
            puttyParams.AppendFormat("{0}@{1}", _ssh.Username, host[0]);
            Process.Start(_config.PuttyPath, puttyParams.ToString());
        }

        private void RunUnix() {
            // TODO
        }
    }
}
